@@ -80,6 +80,14 @@ int    get_disc_status      (int fd, int *empty, int *is_rewritable, int *is_bla
 
 #define CD_ROM_SPEED 176
 
+#define CDS_NO_INFO		0	/* if not implemented */
+#define CDS_AUDIO		100
+#define CDS_DATA_1		101
+#define CDS_DATA_2		102
+#define CDS_XA_2_1		103
+#define CDS_XA_2_2		104
+#define CDS_MIXED		105
+
 static struct {
 	const char *name;
 	gboolean can_write_cdr;
@@ -415,11 +423,11 @@ static int
 get_disc_type (const char *dev_path)
 {
 	int                   fd = 0;
+	int                   type;
 #if defined (HAVE_SYS_MNTTAB_H)   /* Solaris */
 	GString              *new_dev_path;
 	struct cdrom_tocentry entry;
 	struct cdrom_tochdr   header;
-	int                   type;
 
 	/* For ioctl call to work dev_path has to be /vol/dev/rdsk.
 	 * There has to be a nicer way to do this.
@@ -474,6 +482,7 @@ get_disc_type (const char *dev_path)
 	}
 
 	if (ioctl (fd, CDIOREADTOCHEADER, &header) == 0) {
+		close (fd);
 	    	return -1;
 	}
 
@@ -484,7 +493,7 @@ get_disc_type (const char *dev_path)
 		entry.track++) {
 	    	entry.address_format = CD_LBA_FORMAT;
 		if (ioctl (fd, CDIOREADTOCENTRY, &entry) == 0) {
-		    	if (entry.entry.control & CDROM_DATA_TRACK) {
+		    	if (!(entry.entry.control & CDROM_DATA_TRACK)) {
 			    	type = CDS_AUDIO;
 				break;
 			}
@@ -506,6 +515,7 @@ get_disc_type (const char *dev_path)
 	}
 
 #endif /* defined(__FreeBSD__) || defined(__FreeBSD_kernel__) || defined(__DragonFly__)*/
+	close (fd);
 	return type;
 #elif defined(__linux__)
 	fd = open (dev_path, O_RDONLY|O_NONBLOCK);
@@ -517,7 +527,11 @@ get_disc_type (const char *dev_path)
 		fd = -1;
 		return -1;
 	}
-	return ioctl (fd, CDROM_DISC_STATUS, CDSL_CURRENT);
+
+	type = ioctl (fd, CDROM_DISC_STATUS, CDSL_CURRENT);
+	close (fd);
+
+	return type;
 #else /* defined(__linux__) */
 	return CDS_NO_INFO;
 #endif /* defined(__linux__) */
@@ -627,8 +641,6 @@ nautilus_burn_drive_get_media_type_from_path_full (const char *device,
 
 			return NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN;
 		}
-	} else {
-		g_warning ("Could not read MMC profile of %s", device);
 	}
 
 	close_ioctl_handle (ioctl_handle);
